var bettergmail2_preferencesService = null;

// Deletes a preference
function bettergmail2_deletePreference(preference) {
    // If the preference is set
    if(preference) {
        // If a user preference is set
        if(bettergmail2_isPreferenceSet(preference)) {
            bettergmail2_getPreferencesService().clearUserPref(preference);
        }
    }
}

// Deletes a preference branch
function bettergmail2_deletePreferenceBranch(branch) {
    // If the branch is set
    if(branch) {
        bettergmail2_getPreferencesService().deleteBranch(branch);
    }
}

// Gets a boolean preference, returning false if the preference is not set
function bettergmail2_getBooleanPreference(preference, userPreference) {
    // If the preference is set
    if(preference) {
        // If not a user preference or a user preference is set
        if(!userPreference || bettergmail2_isPreferenceSet(preference)) {
            try {
                return bettergmail2_getPreferencesService().getBoolPref(preference);
            } catch(exception) {
                // Do nothing
            }
        }
    }
    
    return false;
}

// Gets an integer preference, returning 0 if the preference is not set
function bettergmail2_getIntegerPreference(preference, userPreference) {
    // If the preference is set
    if(preference) {
        // If not a user preference or a user preference is set
        if(!userPreference || bettergmail2_isPreferenceSet(preference)) {
            try {
                return bettergmail2_getPreferencesService().getIntPref(preference);
            } catch(exception) {
                // Do nothing
            }
        }
    }
    
    return 0;
}

// Gets the preferences service
function bettergmail2_getPreferencesService() {
    // If the preferences service is not set
    if(!bettergmail2_preferencesService) {
        bettergmail2_preferencesService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
    }
    
    return bettergmail2_preferencesService;
}

// Gets a string preference, returning null if the preference is not set
function bettergmail2_getStringPreference(preference, userPreference) {
    // If the preference is set
    if(preference) {
    
        // If not a user preference or a user preference is set
        if(!userPreference || bettergmail2_isPreferenceSet(preference)) {
            try {
            
                return bettergmail2_getPreferencesService().getComplexValue(preference, Components.interfaces.nsISupportsString).data;
            } catch(exception) {
                // Do nothing
                //alert(exception);
            }
        }
    }
    
    return null;
}

// Is a preference set
function bettergmail2_isPreferenceSet(preference) {

    // If the preference is set
    if(preference) {
        return bettergmail2_getPreferencesService().prefHasUserValue(preference);
    }
    
    return false;
}

// Sets a boolean preference
function bettergmail2_setBooleanPreference(preference, value) {
    // If the preference is set
    if(preference) {
        bettergmail2_getPreferencesService().setBoolPref(preference, value);
    }
}

// Sets a boolean preference if it is not already set
function bettergmail2_setBooleanPreferenceIfNotSet(preference, value) {
    // If the preference is not set
    if(!bettergmail2_isPreferenceSet(preference)) {
        bettergmail2_getPreferencesService().setBoolPref(preference, value);
    }
}

// Sets an integer preference
function bettergmail2_setIntegerPreference(preference, value) {
    // If the preference is set
    if(preference) {
        bettergmail2_getPreferencesService().setIntPref(preference, value);
    }
}

// Sets an integer preference if it is not already set
function bettergmail2_setIntegerPreferenceIfNotSet(preference, value) {
    // If the preference is not set
    if(!bettergmail2_isPreferenceSet(preference)) {
        bettergmail2_setIntegerPreference(preference, value);
    }
}

// Sets a string preference
function bettergmail2_setStringPreference(preference, value) {
    // If the preference is set
    if(preference) {
        var supportsStringInterface = Components.interfaces.nsISupportsString;
        var string                  = Components.classes["@mozilla.org/supports-string;1"].createInstance(supportsStringInterface);
        
        string.data = value;
        
        bettergmail2_getPreferencesService().setComplexValue(preference, supportsStringInterface, string);
    }
}

// Sets a string preference if it is not already set
function bettergmail2_setStringPreferenceIfNotSet(preference, value) {
    // If the preference is not set
    if(!bettergmail2_isPreferenceSet(preference)) {
        bettergmail2_setStringPreference(preference, value);
    }
}

function bettergmail2_setStringPreferenceWithForce(preference, value, force) {
	if (force) {
		bettergmail2_setStringPreference(preference, value);
	} else {
		bettergmail2_setStringPreferenceIfNotSet(preference, value); 
	}
}

function bettergmail2_setIntegerPreferenceWithForce(preference, value, force) {
	if (force) {
		bettergmail2_setIntegerPreference(preference, value);
	} else {
		bettergmail2_setIntegerPreferenceIfNotSet(preference, value); 
	}
}

function bettergmail2_setBooleanPreferenceWithForce(preference, value, force) {
	if (force) {
		bettergmail2_setBooleanPreference(preference, value);
	} else {
		bettergmail2_setBooleanPreferenceIfNotSet(preference, value); 
	}
}